@@ -3,7 +3,7 @@ use std::env::var;
 
 fn main() {
     let asm_name = format!(
-        "src/imp/linux_raw/arch/{}.S",
+        "src/imp/linux_raw/arch/outline/{}.S",
         var("CARGO_CFG_TARGET_ARCH").unwrap()
     );
     let os_name = var("CARGO_CFG_TARGET_OS").unwrap();
@@ -13,7 +13,7 @@ fn main() {
     // If posish_use_libc is set, or if we're on an architecture/OS that doesn't
     // have raw syscall support, use libc.
     if var("CARGO_CFG_POSISH_USE_LIBC").is_ok()
-        || !(os_name == "linux")
+        || os_name != "linux"
         || std::fs::metadata(&asm_name).is_err()
         || is_x32
     {
@@ -26,6 +26,7 @@ fn main() {
             .channel
         {
             println!("cargo:rustc-cfg=linux_raw_inline_asm");
+            println!("cargo:rustc-cfg=const_fn_union");
         } else {
             Build::new().file(&asm_name).compile("asm");
             println!("cargo:rerun-if-changed={}", asm_name);
