@@ -2,6 +2,8 @@ mod error;
 mod poll_fd;
 mod types;
 
+#[cfg(any(target_os = "android", target_os = "linux"))]
+pub mod epoll;
 pub use error::Error;
 pub use poll_fd::{PollFd, PollFlags};
 #[cfg(any(
@@ -14,12 +16,12 @@ pub use poll_fd::{PollFd, PollFlags};
 pub use types::PipeFlags;
 #[cfg(all(target_os = "linux", target_env = "gnu"))]
 pub use types::ReadWriteFlags;
-#[cfg(any(target_os = "android", target_os = "linux"))]
-pub use types::UserFaultFdFlags;
 #[cfg(not(any(target_os = "redox", target_os = "wasi")))]
 pub use types::PIPE_BUF;
 #[cfg(not(target_os = "wasi"))]
 pub use types::{DupFlags, MapFlags, ProtFlags, Tcflag, Termios, Winsize, ICANON};
+#[cfg(any(target_os = "android", target_os = "linux"))]
+pub use types::{EventfdFlags, UserfaultfdFlags};
 
 use std::os::raw::c_int;
 
