@@ -13,7 +13,7 @@ use super::{
 };
 use crate::{
     as_mut_ptr, as_ptr, io,
-    io::{AsRawFd, FromRawFd, IntoRawFd, RawFd},
+    io::{AsRawFd, FromRawFd, RawFd},
 };
 use io_lifetimes::{BorrowedFd, OwnedFd};
 #[cfg(target_pointer_width = "64")]
@@ -84,9 +84,9 @@ pub(super) fn borrowed_fd(fd: BorrowedFd<'_>) -> usize {
 }
 
 #[inline]
-pub(super) fn owned_fd(fd: OwnedFd) -> usize {
+pub(super) fn raw_fd(fd: c_int) -> usize {
     // As above, use zero-extension rather than sign-extension.
-    fd.into_raw_fd() as c_uint as usize
+    fd as c_uint as usize
 }
 
 #[inline]
@@ -130,7 +130,7 @@ pub(super) unsafe fn opt_mut<T: Sized>(t: Option<&mut T>) -> usize {
 /// `Option<&T>` is represented as a nullable pointer to `T`, which is the
 /// same size as a `usize`, so we can directly transmute it and pass the result
 /// to syscalls expecting nullable pointers.
-#[cfg(target_arch = "aarch64")]
+#[cfg(any(target_arch = "aarch64", target_arch = "riscv64"))]
 #[inline]
 pub(super) unsafe fn opt_ref<T: Sized>(t: Option<&T>) -> usize {
     transmute(t)
@@ -289,6 +289,18 @@ pub(super) unsafe fn ret_owned_fd(raw: usize) -> io::Result<OwnedFd> {
     }
 }
 
+#[inline]
+pub(super) fn ret_discarded_fd(raw: usize) -> io::Result<()> {
+    if (raw as isize) < 0 {
+        // As above, discourage the optimizer from speculating the `Err`.
+        let raw = suppress_optimization(raw);
+
+        Err(io::Error((raw as u16).wrapping_neg()))
+    } else {
+        Ok(())
+    }
+}
+
 #[inline]
 pub(super) fn ret_void_star(raw: usize) -> io::Result<*mut c_void> {
     check_error(raw)?;
