@@ -1,13 +1,7 @@
 use crate::{imp, io};
 use io_lifetimes::OwnedFd;
 
-#[cfg(any(
-    linux_raw,
-    all(
-        libc,
-        not(any(target_os = "ios", target_os = "macos", target_os = "wasi"))
-    )
-))]
+#[cfg(any(linux_raw, all(libc, not(any(target_os = "ios", target_os = "macos")))))]
 pub use imp::io::PipeFlags;
 
 /// `pipe()`—Creates a pipe.
@@ -21,7 +15,6 @@ pub use imp::io::PipeFlags;
 ///
 /// [POSIX]: https://pubs.opengroup.org/onlinepubs/9699919799/functions/pipe.html
 /// [Linux]: https://man7.org/linux/man-pages/man2/pipe.2.html
-#[cfg(any(target_os = "ios", target_os = "macos"))]
 #[inline]
 pub fn pipe() -> io::Result<(OwnedFd, OwnedFd)> {
     imp::syscalls::pipe()
@@ -36,7 +29,7 @@ pub fn pipe() -> io::Result<(OwnedFd, OwnedFd)> {
 ///  - [Linux]
 ///
 /// [Linux]: https://man7.org/linux/man-pages/man2/pipe2.2.html
-#[cfg(not(any(target_os = "ios", target_os = "macos", target_os = "wasi")))]
+#[cfg(not(any(target_os = "ios", target_os = "macos")))]
 #[inline]
 #[doc(alias = "pipe2")]
 pub fn pipe_with(flags: PipeFlags) -> io::Result<(OwnedFd, OwnedFd)> {
